#include "mainwindow.h"
#include <QIcon>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{
    currentDir = 0;
    DirModel = new QDirModel;
    ListView = new QListView;
    ToolBar = new QToolBar("Navigation");
    StatusBar = new QStatusBar;
    DockWidget = new QDockWidget;
    Label = new QLabel;
    StatusBar->addWidget(Label);
    this->setCentralWidget(ListView);
    this->addToolBar(ToolBar);
    this->setStatusBar(StatusBar);
    this->addDockWidget(Qt::LeftDockWidgetArea,DockWidget);
    this->DockWidget->setWindowTitle("Point to enters");
    homedir="/home";
    Action();
    ListView_config();

    connect(ListView,SIGNAL(doubleClicked(QModelIndex)),this,SLOT(DirList(QModelIndex)));
    connect(ListView,SIGNAL(pressed(QModelIndex)),this,SLOT(viewFileName(QModelIndex)));
    connect(LineEdit,SIGNAL(editingFinished()),this,SLOT(setPathDir()));
    connect(back,SIGNAL(triggered()),this,SLOT(BackDir()));
    connect(next,SIGNAL(triggered()),this,SLOT(NextDir()));
    connect(home,SIGNAL(triggered()),this,SLOT(HomeDir()));
    connect(up,SIGNAL(triggered()),this,SLOT(UpDir()));
}

MainWindow::~MainWindow(){}

//Функции

void MainWindow::ListView_config(){ // настраивает поведение обозревателя {QListView}
    ListView->setModel(DirModel);
    ListView->setAcceptDrops(false);
    this->setAcceptDrops(true);
    DirModel->setSorting(QDir::Name);
    DirModel->sort(0,Qt::AscendingOrder);
    ListView->setViewMode(QListView::IconMode);
    ListView->setResizeMode(QListView::Adjust);
    ListView->setMovement(QListView::Static);
    ListView->setFlow(QListView::LeftToRight);
    ListView->setTextElideMode(Qt::ElideRight);
    ListView->setLayoutMode(QListView::Batched);
    ListView->setDragDropMode(QListView::DragDrop);
    ListView->setSelectionMode(QListView::ExtendedSelection);
    ListView->setGridSize(QSize(110,80));
    ListView->setWordWrap(true);
    ListDir << StringToModelIndex("/");
    viewPath(ListDir.at(0));
    ListView->setRootIndex(ListDir.at(0));
}

void MainWindow::Action(){ // Создает и настраивает кнопки действий
    ToolBar->setIconSize(QSize(30,30));
    back = new QAction(QIcon("icon/back.png"),"Back",this);
    up = new QAction(QIcon("icon/up.png"),"up",this);
    next = new QAction(QIcon("icon/next.png"),"Next",this);
    home = new QAction(QIcon("icon/home.png"),"Home",this);
    LineEdit = new QLineEdit;
    navigateEnabled();
    up->setEnabled(false);
    ToolBar->addAction(back);
    ToolBar->addAction(up);
    ToolBar->addAction(next);
    ToolBar->addAction(home);
    ToolBar->addSeparator();
    ToolBar->addWidget(LineEdit);
}

void MainWindow::EnterDir(QModelIndex dir){
    if (DirModel->isDir(dir)){
        ListView->setRootIndex(dir);
        ListDir = refreshListDir(ListDir, currentDir);
        currentDir++;
        ListDir.insert(currentDir,dir);
        navigateEnabled();
        viewPath(dir);
    }
}

void MainWindow::setActionIndex(int c){ // выполняет действия перехода {вперед} {назад}
    if (currentDir+c >= 0 && currentDir+c <= ListDir.size()-1){
        currentDir = currentDir+c;
    }
    ListView->setRootIndex(ListDir.at(currentDir));
    navigateEnabled();
    viewPath(ListDir.at(currentDir));
}

void MainWindow::navigateEnabled(){ //Устанавливает состояние кнопок перехода {вперед} {назад} {вверх}
    if (currentDir > 0)
        back->setEnabled(true);
    else
        back->setEnabled(false);

    if(currentDir+1 < ListDir.size())
        next->setEnabled(true);
    else
        next->setEnabled(false);

    if(ModelIndexToString(ListView->rootIndex()) != "/")
        up->setEnabled(true);
    else
        up->setEnabled(false);
}

QList<QModelIndex> MainWindow::refreshListDir(QList<QModelIndex> a, int count){ //обнуление списка переходов
    QList<QModelIndex> b;
    for (int i = 0; i <= count; i++)
        b << a.at(i);

    return b;
}

void MainWindow::dragEnterEvent(QDragEnterEvent *event){ // Копировать
         event->acceptProposedAction();
}

void MainWindow::dragMoveEvent(QDragEnterEvent *event){ // Копировать
         event->acceptProposedAction();
}

void MainWindow::dropEvent(QDropEvent *event){ // Вставить
    //event->setDropAction(Qt::CopyAction);
    LineEdit->setText(ModelIndexToString(ListView->indexAt(
            event->pos()
                                                           )));
}

QString MainWindow::ModelIndexToString(QModelIndex name){return DirModel->filePath(name);} //возвращает путь QModelIndex
QModelIndex MainWindow::StringToModelIndex(QString name){return DirModel->index(name);} //возвращает QModelIndex пути
void MainWindow::viewPath(QModelIndex name){LineEdit->setText(ModelIndexToString(name));} //Устанавливает {строку вооода}

//Слоты

void MainWindow::BackDir(){setActionIndex(-1);} // переход назад
void MainWindow::NextDir(){setActionIndex(1);} // переход вперед
void MainWindow::HomeDir(){EnterDir(StringToModelIndex(homedir));} // переход в домашний каталог
void MainWindow::UpDir(){EnterDir(DirModel->parent(ListView->rootIndex()));} // переход в родительский каталог

void MainWindow::setPathDir(){EnterDir(StringToModelIndex(LineEdit->text()));}   //Выполняет переход в директорию по {строке ввода}
void MainWindow::viewFileName(QModelIndex name){Label->setText(DirModel->fileName(name));} // Выводит Имя обьекта в строке состояния
void MainWindow::DirList(QModelIndex dir){EnterDir(dir);}   // Выполняет переход в директорию если обьек директория
